'use strict';

var cssnano = require('cssnano');
var lessOpenUI5 = require('less-openui5');
var Chunk = require('webpack/lib/Chunk');

var _require = require('webpack-sources'),
    OriginalSource = _require.OriginalSource;

var NullFactory = require('webpack/lib/NullFactory');
var LocalModuleDependency = require('webpack/lib/dependencies/LocalModuleDependency');
var ConstDependency = require('webpack/lib/dependencies/ConstDependency');
var ContextElementDependency = require('webpack/lib/dependencies/ContextElementDependency');
var OpenUI5DefineDependency = require('./OpenUI5DefineDependency');
var OpenUI5DefineDependencyParserPlugin = require('./OpenUI5DefineDependencyParserPlugin');
var OpenUI5LazyInstanceDependency = require('./OpenUI5LazyInstanceDependency');
var OpenUI5ViewDependency = require('./OpenUI5ViewDependency');
var OpenUI5RequireDependencyParserPlugin = require('./OpenUI5RequireDependencyParserPlugin');
var OpenUI5RequireItemDependency = require('./OpenUI5RequireItemDependency');
var OpenUI5RequireContextDependency = require('./OpenUI5RequireContextDependency');
var OpenUI5ResourceDependencyParserPlugin = require('./OpenUI5ResourceDependencyParserPlugin');
var OpenUI5ResourceDependency = require('./OpenUI5ResourceDependency');
var OpenUI5ResourceModuleFactory = require('./OpenUI5ResourceModuleFactory');

class OpenUI5Plugin {
  constructor(options) {
    this.options = options || {};
  }

  apply(compiler) {
    var options = this.options;


    var resourceModuleFactory = new OpenUI5ResourceModuleFactory(compiler.resolverFactory);

    compiler.hooks.compilation.tap('OpenUI5Plugin', function (compilation, _ref) {
      var normalModuleFactory = _ref.normalModuleFactory,
          contextModuleFactory = _ref.contextModuleFactory;

      compilation.dependencyFactories.set(OpenUI5RequireItemDependency, normalModuleFactory);
      compilation.dependencyTemplates.set(OpenUI5RequireItemDependency, new OpenUI5RequireItemDependency.Template());

      compilation.dependencyFactories.set(OpenUI5RequireContextDependency, contextModuleFactory);
      compilation.dependencyTemplates.set(OpenUI5RequireContextDependency, new OpenUI5RequireContextDependency.Template());

      compilation.dependencyFactories.set(OpenUI5DefineDependency, new NullFactory());
      compilation.dependencyTemplates.set(OpenUI5DefineDependency, new OpenUI5DefineDependency.Template());

      compilation.dependencyFactories.set(OpenUI5LazyInstanceDependency, normalModuleFactory);
      compilation.dependencyTemplates.set(OpenUI5LazyInstanceDependency, new OpenUI5LazyInstanceDependency.Template());

      compilation.dependencyFactories.set(OpenUI5ResourceDependency, resourceModuleFactory);
      compilation.dependencyTemplates.set(OpenUI5ResourceDependency, new OpenUI5ResourceDependency.Template());

      compilation.dependencyFactories.set(OpenUI5ViewDependency, normalModuleFactory);
      compilation.dependencyTemplates.set(OpenUI5ViewDependency, new OpenUI5ViewDependency.Template());

      compilation.dependencyFactories.set(LocalModuleDependency, new NullFactory());
      compilation.dependencyTemplates.set(LocalModuleDependency, new LocalModuleDependency.Template());

      compilation.dependencyFactories.set(ConstDependency, new NullFactory());
      compilation.dependencyTemplates.set(ConstDependency, new ConstDependency.Template());

      compilation.dependencyFactories.set(ContextElementDependency, normalModuleFactory);
      compilation.dependencyFactories.set(ContextElementDependency, normalModuleFactory);

      normalModuleFactory.hooks.parser.for('javascript/auto').tap('OpenUI5Plugin', function (parser) {
        new OpenUI5RequireDependencyParserPlugin(options).apply(parser);
        new OpenUI5DefineDependencyParserPlugin(options).apply(parser);
        new OpenUI5ResourceDependencyParserPlugin(options).apply(parser);
      });
    });

    if (options.theme) {
      compiler.hooks.thisCompilation.tap('OpenUI5Plugin', function (compilation) {
        compilation.hooks.additionalAssets.tapAsync('OpenUI5Plugin', function (callback) {
          var builder = new lessOpenUI5.Builder();
          var promises = [];

          var chunk = new Chunk('openui5_theme');
          chunk.ids = [];

          var _loop = function _loop(lib) {
            var libPath = `${lib.replace(/\./g, '/')}/themes/${options.theme}`;
            var promise = builder.build({
              lessInputPath: `${libPath}/library.source.less`,
              rootPaths: options.rootPaths,
              library: {
                name: lib
              }
            }).then(function (result) {
              return cssnano.process(result.css, {
                postcssNormalizeUrl: false
              }).then(function (result) {
                var file = `${libPath}/library.css`;
                compilation.assets[file] = new OriginalSource(result.css, file);
                chunk.files.push(file);
              });
            }).catch(callback);

            promises.push(promise);
          };

          var _iteratorNormalCompletion = true;
          var _didIteratorError = false;
          var _iteratorError = undefined;

          try {
            for (var _iterator = options.libs[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
              var lib = _step.value;

              _loop(lib);
            }
          } catch (err) {
            _didIteratorError = true;
            _iteratorError = err;
          } finally {
            try {
              if (!_iteratorNormalCompletion && _iterator.return) {
                _iterator.return();
              }
            } finally {
              if (_didIteratorError) {
                throw _iteratorError;
              }
            }
          }

          Promise.all(promises).then(function () {
            compilation.chunks.push(chunk);
            callback();
          }).catch(callback);
        });
      });
    }
  }
}
module.exports = OpenUI5Plugin;